# AGRII taxonomic conversion
- Converting highest resolution taxonomic ID to NCBI's taxaID
- Purpose: To use [genomesizeR](https://github.com/ScionResearch/genomesizeR) to get genome sizes for AGRII dataset
- Example [input](https://github.com/ScionResearch/genomesizeR/blob/main/inst/extdata/example_input.csv) for genomesizeR
- `seqtab_16s.csv` and `TaxaID16s.csv` files received from TG and uploaded via mobaxterm

## Running taxon conversion
```bash
cd /ssd0/susbus/agrii
conda create -n multitax -c bioconda multitax pytaxonkit pandas
conda activate multitax # also includes [pytaxonkit](https://github.com/bioforensics/pytaxonkit)

# setting up the taxonkit library
wget -c ftp://ftp.ncbi.nih.gov/pub/taxonomy/taxdump.tar.gz 
tar -zxvf taxdump.tar.gz

mkdir -p $HOME/.taxonkit
cp names.dmp nodes.dmp delnodes.dmp merged.dmp $HOME/.taxonkit
```

## Parallelised verion to get missing taxaIDs
 - Running a parallelised version to get taxaIDs for first non-NAN value for SILVA taxonomy
 - if NAs are found in pytaxonkit.name2id(), then the next best taxonomic classification is used for taxaID retrieval
 ## Running TG's taxa file to get taxaID
```bash
python

import pytaxonkit, os
import pandas as pd
from concurrent.futures import ThreadPoolExecutor

# reading in the 16S taxa
new_agrii_tax = pd.read_csv("TaxaId16s.csv", header = 0)

# dropping the unnamed column
new_agrii_tax = new_agrii_tax.drop(columns=['Unnamed: 0'])

# Rename the 'ASVrank' column to 'ASV'
new_agrii_tax = new_agrii_tax.rename(columns={'ASVrank': 'ASV'})

# Move 'ASV' to the first column
cols = ['ASV'] + [col for col in new_agrii_tax.columns if col != 'ASV']
new_agrii_tax = new_agrii_tax[cols]

# To verify the result
print(new_agrii_tax.head())
```

## Keeping the last non-NA value as 'name' in the dataframe
```bash
# Create a new column 'name' by finding the last non-NaN value in each row
new_agrii_tax['name'] = new_agrii_tax[['species', 'genus', 'family', 'order', 'class', 'phylum', 'domain']].bfill(axis=1).iloc[:, 0]

# To verify the result
print(new_agrii_tax[['ASV', 'name']].head())

## Running [`pytaxonkit`](https://github.com/bioforensics/pytaxonkit) to get NCBI taxaIDs
# Replace NaN values in the 'name' column with 'unclassified'
new_agrii_tax['name'].fillna('unclassified', inplace=True)

# Extract the 'name' column from your DataFrame
new_names = new_agrii_tax['name'].tolist()

# Run pytaxonkit.name2taxid with the names
new_taxid_results = pytaxonkit.name2taxid(new_names)

# To view the results
print(new_taxid_results)

# Merging new_taxid_results into new_agrii_tax
merged_df = pd.merge(new_agrii_tax, new_taxid_results, left_on='name', right_on='Name', how='left')

# Optionally, you can drop the 'Name' column from the merged DataFrame if you don't need it
merged_df = merged_df.drop(columns=['Name'])

# Remove duplicate rows based on all columns
merged_df = merged_df.drop_duplicates(subset=['ASV'])

# Display the merged DataFrame
print(merged_df)
```

## Finding missing taxaIDs based on column to the left for missing ones
```bash
# Function to find TaxID with early exit after finding the first valid TaxID with verbose output
def get_taxid(row):
    asv_id = row['ASV']  # Get the ASV identifier
    taxid_value = row['TaxID']
    # List of columns to search from most specific (species) to least specific (domain)
    columns_to_search = ['species', 'genus', 'family', 'order', 'class', 'phylum', 'domain']
    # Check if TaxID is missing
    if pd.isna(taxid_value):
        # Start by trying to resolve TaxID for each taxonomic rank in the list
        for col in columns_to_search:
            column_value = row[col]
            # Skip if the value is NaN or not a string
            if pd.isna(column_value) or not isinstance(column_value, str):
                print(f"Skipping invalid value for ASV: {asv_id}, Column: {col}")
                continue
            print(f"Processing ASV: {asv_id}, Column: {col}, Value: {column_value}")
            # Use pytaxonkit to look up TaxID for the column value
            try:
                result = pytaxonkit.name2taxid([column_value])
                # If a valid result is found, return the TaxID
                if isinstance(result, pd.DataFrame) and not result.empty:
                    taxid = result['TaxID'].iloc[0]  # Get the first TaxID
                    if pd.notna(taxid):
                        print(f"Found TaxID: {taxid} for ASV: {asv_id}, Value: {column_value}")
                        return taxid  # Exit once a valid TaxID is found
            except Exception as e:
                print(f"Error processing {column_value}: {e}")
        
        print(f"No valid TaxID found for ASV: {asv_id}, assigning placeholder.")
        return "TaxID_not_found"
    else:
        print(f"Existing TaxID for ASV: {asv_id}: {taxid_value}")
        return taxid_value

# Function for parallel processing using ThreadPoolExecutor
def parallel_taxid_lookup(df):
    with ThreadPoolExecutor() as executor:
        # Apply get_taxid function to each row in the dataframe
        new_taxids = list(executor.map(get_taxid, [row for _, row in df.iterrows()]))
    return new_taxids

# Assuming merged_df is the DataFrame containing the full data with some missing TaxIDs
# Step 1: Filter the DataFrame for rows where TaxID is missing
missing_taxid_df = merged_df[merged_df['TaxID'].isna()]

# Step 2: Run the parallel lookup only for rows with missing TaxIDs
missing_taxid_results = parallel_taxid_lookup(missing_taxid_df)

# Step 3: Update only the rows with missing TaxIDs in the original DataFrame
merged_df['TaxID'] = merged_df['TaxID'].astype(object)
merged_df.loc[merged_df['TaxID'].isna(), 'TaxID'] = missing_taxid_results

# Step 4: Display the updated DataFrame (optional)
print(merged_df)

# Step 5: Save the updated DataFrame with both the new and old TaxIDs to a file
output_file = "agrii_pytaxonkit_taxaIDs_complete.csv"
merged_df.to_csv(output_file, header=True, index=False)

print(f"Updated DataFrame with filled TaxIDs has been saved to {output_file}")

```

## Convert the taxID file to [genomesizeR](https://github.com/ScionResearch/genomesizeR) format
```bash
# Reading in the species file to get counts
species = pd.read_csv("seqtab_16s.csv", header = 0)

# Replace 'Unnamed: 0' with 'ASV_1', 'ASV_2', etc.
species['Unnamed: 0'] = ['ASV_' + str(i + 1) for i in range(len(species))]

# View the result
species.head()

# renaming the ASV column
species = species.rename(columns={'Unnamed: 0': 'ASV'})

# Merge the two dataframes on the 'ASV' column
merged_combined = pd.merge(merged_df, species, on='ASV')

# Drop columns from 'domain' to 'name'
merged_combined = merged_combined.drop(columns=merged_combined.loc[:, 'domain':'species'].columns)

# Rename 'name' to SCIENTIFIC_NAME and TaxID to TAXID
merged_combined = merged_combined.rename(columns={'TaxID': 'TAXID', 'name':'SCIENTIFIC_NAME'})

# View the result
merged_combined.head()

# Saving the file
merged_combined.to_csv("agrii_counts_combined.csv", header=True, index=False)

# Converting the file to long format required for `genomesizeR`
# https://github.com/ScionResearch/genomesizeR/blob/main/inst/extdata/example_input.csv
## Drop the 'ASV' column entirely
merged_combined_no_asv = merged_combined.drop(columns=['ASV'])

# Melt the dataframe to transform it from wide to long format
long_df = pd.melt(
    merged_combined_no_asv, 
    id_vars=['SCIENTIFIC_NAME', 'TAXID'],  # Only keep 'SCIENTIFIC_NAME' and 'TaxID' as identifiers
    var_name='SAMPLE', 
    value_name='COUNT'
)

# Filter out rows with zero counts
long_df = long_df[long_df['COUNT'] > 0]

# Reorder the columns to match the desired output format
long_df = long_df[['COUNT', 'SAMPLE', 'SCIENTIFIC_NAME', 'TAXID']]

# Group by SAMPLE, SCIENTIFIC_NAME AND TAXID and merge counts
grouped_df = long_df.groupby(['SAMPLE', 'SCIENTIFIC_NAME', 'TAXID'], as_index=False)['COUNT'].sum()
grouped_df = grouped_df[['COUNT', 'SAMPLE', 'SCIENTIFIC_NAME', 'TAXID']]

# Save the long format to file
grouped_df.to_csv("agrii_counts_combined_long.csv", header=True, index=False)

```

## Running [`genomesizeR`](https://github.com/ScionResearch/genomesizeR/tree/main)
    - download the `genomesizeR database` from Zenodo: https://zenodo.org/records/13733781/files/genomesizeRdata.tar.gz
    - save to a folder of choice {to be used with `refdata_archive_path`}
```bash
conda activate multitax
R
# dependencies
library(tidyverse)

# reading in the new input file
agrii <- read.csv("agrii_counts_combined_long.csv")

# downloading and installing the packages
remotes::install_github("ScionResearch/genomesizeR")
library(genomesizeR)

# setting the database path
refdata_archive_path = "/hdd0/susbus/databases/genomesizeR/genomesizeRdata.tar.gz"

# test run
example_input_file = system.file("extdata", "example_input.csv", package = "genomesizeR")
results = estimate_genome_size(example_input_file, refdata_archive_path, 
    sep='\t', 
    match_column='TAXID', 
    output_format='input',
    n_cores = 64
)

plotted_df = plot_genome_size_histogram(results)

# full run
agrii_filepath = "/ssd0/susbus/agrii/agrii_counts_combined_long.csv"

agrii_results = estimate_genome_size(agrii_filepath, refdata_archive_path, 
    sep=',', 
    match_column='TAXID', 
    output_format='input',
    n_cores = 64
)

# write the full results to file
write.csv(agrii_results, "agrii_genomesizeR_full_output.csv")

# merging with original data
# keeping only important columns
agrii_counts <- read.csv("agrii_counts_combined.csv") %>% select(ASV, SCIENTIFIC_NAME, TAXID)

# merging with original
agrii_updated = agrii_counts %>% mutate(TAXID=as.character(TAXID),
                           TAXID = str_replace_all(TAXID, "\\s+", ""))

agrii_results_select = agrii_results %>% select(SCIENTIFIC_NAME, TAXID, estimated_genome_size)

# Group by SCIENTIFIC_NAME and TAXID, calculate the mean, and round to the nearest whole number
agrii_results_summary <- agrii_results_select %>%
  group_by(SCIENTIFIC_NAME, TAXID) %>%
  summarise(mean_genome_size = round(mean(estimated_genome_size, na.rm = TRUE)))

# View the result
head(agrii_results_summary)

# merging the results
agrii_merged = left_join(agrii_updated, agrii_results_summary, by = c("SCIENTIFIC_NAME")) %>%
                select(-`TAXID.y`) %>%
                rename("TAXID"="TAXID.x")

head(agrii_merged)

# Calculate the mean of the mean_genome_size column, excluding NA values
column_mean <- mean(agrii_merged$mean_genome_size, na.rm = TRUE)

# Replace NA values with the column mean
agrii_merged_final <- agrii_merged %>%
  mutate(mean_genome_size = ifelse(is.na(mean_genome_size), column_mean, mean_genome_size))

# View the updated result
head(agrii_merged_final)
tail(agrii_merged_final)

# writing the merged final results to file
write.csv(agrii_merged_final, "agrii_genomesizeR_final_output.csv")

```

## SOCIAL NICHE BREADTH
- [Social Niche Breadth](https://www.nature.com/articles/s41559-023-02027-7#Sec2) analyses 
- [SNB_repo](https://github.com/MGXlab/social_niche_breadth_SNB)

```bash
conda activate multitax
cd /ssd0/susbus/agrii && mkdir snb
cd /ssd0/susbus/agrii/snb

# got the `microeco_decostand_seqtab.csv` files from TG and added via mobaxterm to the folder
# also edited the microeco_rarefied_taxa.csv file by removing the ASVrank column and saving to `edited_microeco_rarefied_taxa.csv`

# merging taxa and percentage files
python merge_taxa_decostand.py edited_microeco_rarefied_taxa.csv microeco_decostand_seqtab.csv agrii_taxa_decostand.csv
python merge_taxonomy_final.py agrii_taxa_decostand.csv snb_input_table

# running SNB on relative abundances
conda activate sourmash
./calculate_SNB.py -f snb_input_table.txt -o agrii_snb_output.txt -r species --c1 0.001 --c2 0.001 -m --force

```

## rRNA copy number prediction
- ASV sequences were used to predict rRNA copy number with [ANNA16](https://github.com/Merlinaphist/ReproduceANNA16)
- Sequences were run on the [Google colab notebook](https://colab.research.google.com/drive/1XwpTMCHSfTmzpHyKrmiD8aC8C_1nndUV#scrollTo=V_Fe5x8g8sCv)


# running SNB with raw counts
python merge_taxa_decostand.py edited_microeco_rarefied_taxa.csv microeco_converted.csv agrii_taxa_rarefied.csv
python merge_taxonomy_final.py agrii_taxa_rarefied.csv snb_input_table_raw

conda activate sourmash
# running SNB on raw counts
# note: open the snb_input_table_raw.txt file and make sure the last column has the total number of reads in brackets, i.e. (7555)
./calculate_SNB.py -f snb_input_table_raw.txt -o agrii_raw_snb_output.txt --c1 0 --c2 0 -r class --force
